#
# Copyright (c) 2001 - 2014 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "src/engine/SCons/Memoize.py  2014/09/27 12:51:43 garyo"

__doc__ = """Memoizer

A metaclass implementation to count hits and misses of the computed
values that various methods cache in memory.

Use of this modules assumes that wrapped methods be coded to cache their
values in a consistent way.  Here is an example of wrapping a method
that returns a computed value, with no input parameters:

    memoizer_counters = []                                      # Memoization

    memoizer_counters.append(SCons.Memoize.CountValue('foo'))   # Memoization

    def foo(self):

        try:                                                    # Memoization
            return self._memo['foo']                            # Memoization
        except KeyError:                                        # Memoization
            pass                                                # Memoization

        result = self.compute_foo_value()

        self._memo['foo'] = result                              # Memoization

        return result

Here is an example of wrapping a method that will return different values
based on one or more input arguments:

    def _bar_key(self, argument):                               # Memoization
        return argument                                         # Memoization

    memoizer_counters.append(SCons.Memoize.CountDict('bar', _bar_key)) # Memoization

    def bar(self, argument):

        memo_key = argument                                     # Memoization
        try:                                                    # Memoization
            memo_dict = self._memo['bar']                       # Memoization
        except KeyError:                                        # Memoization
            memo_dict = {}                                      # Memoization
            self._memo['dict'] = memo_dict                      # Memoization
        else:                                                   # Memoization
            try:                                                # Memoization
                return memo_dict[memo_key]                      # Memoization
            except KeyError:                                    # Memoization
                pass                                            # Memoization

        result = self.compute_bar_value(argument)

        memo_dict[memo_key] = result                            # Memoization

        return result

At one point we avoided replicating this sort of logic in all the methods
by putting it right into this module, but we've moved away from that at
present (see the "Historical Note," below.).

Deciding what to cache is tricky, because different configurations
can have radically different performance tradeoffs, and because the
tradeoffs involved are often so non-obvious.  Consequently, deciding
whether or not to cache a given method will likely be more of an art than
a science, but should still be based on available data from this module.
Here are some VERY GENERAL guidelines about deciding whether or not to
cache return values from a method that's being called a lot:

    --  The first question to ask is, "Can we change the calling code
        so this method isn't called so often?"  Sometimes this can be
        done by changing the algorithm.  Sometimes the *caller* should
        be memoized, not the method you're looking at.

    --  The memoized function should be timed with multiple configurations
        to make sure it doesn't inadvertently slow down some other
        configuration.

    --  When memoizing values based on a dictionary key composed of
        input arguments, you don't need to use all of the arguments
        if some of them don't affect the return values.

Historical Note:  The initial Memoizer implementation actually handled
the caching of values for the wrapped methods, based on a set of generic
algorithms for computing hashable values based on the method's arguments.
This collected caching logic nicely, but had two drawbacks:

    Running arguments through a generic key-conversion mechanism is slower
    (and less flexible) than just coding these things directly.  Since the
    methods that need memoized values are generally performance-critical,
    slowing them down in order to collect the logic isn't the right
    tradeoff.

    Use of the memoizer really obscured what was being called, because
    all the memoized methods were wrapped with re-used generic methods.
    This made it more difficult, for example, to use the Python profiler
    to figure out how to optimize the underlying methods.
"""

import types

# A flag controlling whether or not we actually use memoization.
use_memoizer = None

CounterList = []

class Counter(object):
    """
    Base class for counting memoization hits and misses.

    We expect that the metaclass initialization will have filled in
    the .name attribute that represents the name of the function
    being counted.
    """
    def __init__(self, method_name):
        """
        """
        self.method_name = method_name
        self.hit = 0
        self.miss = 0
        CounterList.append(self)
    def display(self):
        fmt = "    %7d hits %7d misses    %s()"
        print fmt % (self.hit, self.miss, self.name)
    def __cmp__(self, other):
        try:
            return cmp(self.name, other.name)
        except AttributeError:
            return 0

class CountValue(Counter):
    """
    A counter class for simple, atomic memoized values.

    A CountValue object should be instantiated in a class for each of
    the class's methods that memoizes its return value by simply storing
    the return value in its _memo dictionary.

    We expect that the metaclass initialization will fill in the
    .underlying_method attribute with the method that we're wrapping.
    We then call the underlying_method method after counting whether
    its memoized value has already been set (a hit) or not (a miss).
    """
    def __call__(self, *args, **kw):
        obj = args[0]
        if self.method_name in obj._memo:
            self.hit = self.hit + 1
        else:
            self.miss = self.miss + 1
        return self.underlying_method(*args, **kw)

class CountDict(Counter):
    """
    A counter class for memoized values stored in a dictionary, with
    keys based on the method's input arguments.

    A CountDict object is instantiated in a class for each of the
    class's methods that memoizes its return value in a dictionary,
    indexed by some key that can be computed from one or more of
    its input arguments.

    We expect that the metaclass initialization will fill in the
    .underlying_method attribute with the method that we're wrapping.
    We then call the underlying_method method after counting whether the
    computed key value is already present in the memoization dictionary
    (a hit) or not (a miss).
    """
    def __init__(self, method_name, keymaker):
        """
        """
        Counter.__init__(self, method_name)
        self.keymaker = keymaker
    def __call__(self, *args, **kw):
        obj = args[0]
        try:
            memo_dict = obj._memo[self.method_name]
        except KeyError:
            self.miss = self.miss + 1
        else:
            key = self.keymaker(*args, **kw)
            if key in memo_dict:
                self.hit = self.hit + 1
            else:
                self.miss = self.miss + 1
        return self.underlying_method(*args, **kw)

class Memoizer(object):
    """Object which performs caching of method calls for its 'primary'
    instance."""

    def __init__(self):
        pass

def Dump(title=None):
    if title:
        print title
    CounterList.sort()
    for counter in CounterList:
        counter.display()

class Memoized_Metaclass(type):
    def __init__(cls, name, bases, cls_dict):
        super(Memoized_Metaclass, cls).__init__(name, bases, cls_dict)

        for counter in cls_dict.get('memoizer_counters', []):
            method_name = counter.method_name

            counter.name = cls.__name__ + '.' + method_name
            counter.underlying_method = cls_dict[method_name]

            replacement_method = types.MethodType(counter, None, cls)
            setattr(cls, method_name, replacement_method)

def EnableMemoization():
    global use_memoizer
    use_memoizer = 1

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



"""SCons.Tool.Packaging.tarbz2

The tarbz2 SRC packager.
"""

#
# Copyright (c) 2001 - 2014 The SCons Foundation
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "src/engine/SCons/Tool/packaging/src_tarbz2.py  2014/09/27 12:51:43 garyo"

from SCons.Tool.packaging import putintopackageroot

def package(env, target, source, PACKAGEROOT, **kw):
    bld = env['BUILDERS']['Tar']
    bld.set_suffix('.tar.bz2')
    target, source = putintopackageroot(target, source, env, PACKAGEROOT, honor_install_location=0)
    return bld(env, target, source, TARFLAGS='-jc')

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



"""SCons.Tool.CVS.py

Tool-specific initialization for CVS.

There normally shouldn't be any need to import this module directly.
It will usually be imported through the generic SCons.Tool.Tool()
selection method.

"""

# Copyright (c) 2001 - 2014 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

__revision__ = "src/engine/SCons/Tool/CVS.py  2014/09/27 12:51:43 garyo"

import SCons.Action
import SCons.Builder
import SCons.Util

def generate(env):
    """Add a Builder factory function and construction variables for
    CVS to an Environment."""

    def CVSFactory(repos, module='', env=env):
        """ """
        import SCons.Warnings as W
        W.warn(W.DeprecatedSourceCodeWarning, """The CVS() factory is deprecated and there is no replacement.""")
        # fail if repos is not an absolute path name?
        if module != '':
           # Don't use os.path.join() because the name we fetch might
           # be across a network and must use POSIX slashes as separators.
           module = module + '/'
           env['CVSCOM']   = '$CVS $CVSFLAGS co $CVSCOFLAGS -d ${TARGET.dir} $CVSMODULE${TARGET.posix}'
        act = SCons.Action.Action('$CVSCOM', '$CVSCOMSTR')
        return SCons.Builder.Builder(action = act,
                                     env = env,
                                     CVSREPOSITORY = repos,
                                     CVSMODULE = module)

    #setattr(env, 'CVS', CVSFactory)
    env.CVS = CVSFactory

    env['CVS']        = 'cvs'
    env['CVSFLAGS']   = SCons.Util.CLVar('-d $CVSREPOSITORY')
    env['CVSCOFLAGS'] = SCons.Util.CLVar('')
    env['CVSCOM']     = '$CVS $CVSFLAGS co $CVSCOFLAGS ${TARGET.posix}'

def exists(env):
    return env.Detect('cvs')

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#
# Copyright (c) 2001 - 2014 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
NOTE: Installed SCons is not importable like usual Python packages. It is
      executed explicitly with command line scripts. This allows multiple
      SCons versions to coexist within single Python installation, which
      is critical for enterprise build cases. Explicit invokation is
      necessary to avoid confusion over which version of SCons is active.

      By default SCons is installed into versioned directory, e.g.
      site-packages/scons-2.1.0.alpha.20101125 and much of the stuff
      below is dedicated to make it happen on various platforms.
"""

__revision__ = "src/setup.py  2014/09/27 12:51:43 garyo"

import os
import stat
import sys

Version = "2.3.4"

man_pages = [
    'scons.1',
    'sconsign.1',
    'scons-time.1',
]

# Exit with error if trying to install with Python >= 3.0
if sys.version_info >= (3,0,0):
    msg = "scons: *** SCons does not run under Python version %s.\n\
Python 3 and above are not yet supported.\n"
    sys.stderr.write(msg % (sys.version.split()[0]))
    sys.exit(1)


# change to setup.py directory if it was executed from other dir
(head, tail) = os.path.split(sys.argv[0])
if head:
    os.chdir(head)
    sys.argv[0] = tail


# flag if setup.py is run on win32 or _for_ win32 platform,
# (when building windows installer on linux, for example)
is_win32 = 0
if not sys.platform == 'win32':
    try:
        if sys.argv[1] == 'bdist_wininst':
            is_win32 = 1
    except IndexError:
        pass
else:
    is_win32 = 1


import distutils
import distutils.core
import distutils.command.install
import distutils.command.install_data
import distutils.command.install_lib
import distutils.command.install_scripts
import distutils.command.build_scripts

_install = distutils.command.install.install
_install_data = distutils.command.install_data.install_data
_install_lib = distutils.command.install_lib.install_lib
_install_scripts = distutils.command.install_scripts.install_scripts
_build_scripts = distutils.command.build_scripts.build_scripts

class _options(object):
    pass

Options = _options()

Installed = []

def set_explicitly(name, args):
    """
    Return if the installation directory was set explicitly by the
    user on the command line.  This is complicated by the fact that
    "install --install-lib=/foo" gets turned into "install_lib
    --install-dir=/foo" internally.
    """
    if args[0] == "install_" + name:
        s = "--install-dir="
    else:
        # The command is something else (usually "install")
        s = "--install-%s=" % name
    set = 0
    length = len(s)
    for a in args[1:]:
        if a[:length] == s:
            set = 1
            break
    return set

class install(_install):
    user_options = _install.user_options + [
                    ('no-scons-script', None,
                     "don't install 'scons', only install 'scons-%s'" % Version),
                    ('no-version-script', None,
                     "don't install 'scons-%s', only install 'scons'" % Version),
                    ('install-bat', None,
                     "install 'scons.bat' script"),
                    ('no-install-bat', None,
                     "do not install 'scons.bat' script"),
                    ('install-man', None,
                     "install SCons man pages"),
                    ('no-install-man', None,
                     "do not install SCons man pages"),
                    ('standard-lib', None,
                     "install SCons library in standard Python location"),
                    ('standalone-lib', None,
                     "install SCons library in separate standalone directory"),
                    ('version-lib', None,
                     "install SCons library in version-numbered directory"),
                   ]
    boolean_options = _install.boolean_options + [
                       'no-scons-script',
                       'no-version-script',
                       'install-bat',
                       'no-install-bat',
                       'install-man',
                       'no-install-man',
                       'standard-lib',
                       'standalone-lib',
                       'version-lib'
                      ]

    if hasattr(os, 'link'):
        user_options.append(
                    ('hardlink-scons', None,
                     "hard link 'scons' to the version-numbered script, don't make a separate 'scons' copy"),
                     )
        boolean_options.append('hardlink-script')

    if hasattr(os, 'symlink'):
        user_options.append(
                    ('symlink-scons', None,
                     "make 'scons' a symbolic link to the version-numbered script, don't make a separate 'scons' copy"),
                     )
        boolean_options.append('symlink-script')

    def initialize_options(self):
        _install.initialize_options(self)
        self.no_scons_script = 0
        self.no_version_script = 0
        self.install_bat = 0
        self.no_install_bat = not is_win32
        self.install_man = 0
        self.no_install_man = is_win32
        self.standard_lib = 0
        self.standalone_lib = 0
        self.version_lib = 0
        self.hardlink_scons = 0
        self.symlink_scons = 0
        # Don't warn about having to put the library directory in the
        # search path.
        self.warn_dir = 0

    def finalize_options(self):
        _install.finalize_options(self)
        if self.install_bat:
            Options.install_bat = 1
        else:
            Options.install_bat = not self.no_install_bat
        if self.install_man:
            Options.install_man = 1
        else:
            Options.install_man = not self.no_install_man
        Options.standard_lib = self.standard_lib
        Options.standalone_lib = self.standalone_lib
        Options.version_lib = self.version_lib
        Options.install_scons_script = not self.no_scons_script
        Options.install_version_script = not self.no_version_script
        Options.hardlink_scons = self.hardlink_scons
        Options.symlink_scons = self.symlink_scons

def get_scons_prefix(libdir, is_win32):
    """
    Return the right prefix for SCons library installation.  Find
    this by starting with the library installation directory
    (.../site-packages, most likely) and crawling back up until we reach
    a directory name beginning with "python" (or "Python").
    """
    drive, head = os.path.splitdrive(libdir)
    while head:
        if head == os.sep:
            break
        head, tail = os.path.split(head)
        if tail.lower()[:6] == "python":
            # Found the Python library directory...
            if is_win32:
                # ...on Win32 systems, "scons" goes in the directory:
                #    C:\PythonXX => C:\PythonXX\scons
                return os.path.join(drive + head, tail)
            else:
                # ...on other systems, "scons" goes above the directory:
                #    /usr/lib/pythonX.X => /usr/lib/scons
                return os.path.join(drive + head)
    return libdir

def force_to_usr_local(self):
    """
    A hack to decide if we need to "force" the installation directories
    to be under /usr/local.  This is because Mac Os X Tiger and
    Leopard, by default, put the libraries and scripts in their own
    directories under /Library or /System/Library.
    """
    return (sys.platform[:6] == 'darwin' and
            (self.install_dir[:9] == '/Library/' or
             self.install_dir[:16] == '/System/Library/'))

class install_lib(_install_lib):
    def finalize_options(self):
        _install_lib.finalize_options(self)
        if force_to_usr_local(self):
            self.install_dir = '/usr/local/lib'
        args = self.distribution.script_args
        if not set_explicitly("lib", args):
            # They didn't explicitly specify the installation
            # directory for libraries...
            is_win32 = sys.platform == "win32" or args[0] == 'bdist_wininst'
            prefix = get_scons_prefix(self.install_dir, is_win32)
            if Options.standalone_lib:
                # ...but they asked for a standalone directory.
                self.install_dir = os.path.join(prefix, "scons")
            elif Options.version_lib or not Options.standard_lib:
                # ...they asked for a version-specific directory,
                # or they get it by default.
                self.install_dir = os.path.join(prefix, "scons-%s" % Version)

        msg = "Installed SCons library modules into %s" % self.install_dir
        Installed.append(msg)

class install_scripts(_install_scripts):
    def finalize_options(self):
        _install_scripts.finalize_options(self)
        if force_to_usr_local(self):
            self.install_dir = '/usr/local/bin'
        self.build_dir = os.path.join('build', 'scripts')
        msg = "Installed SCons scripts into %s" % self.install_dir
        Installed.append(msg)

    def do_nothing(self, *args, **kw):
        pass

    def hardlink_scons(self, src, dst, ver):
        try: os.unlink(dst)
        except OSError: pass
        os.link(ver, dst)

    def symlink_scons(self, src, dst, ver):
        try: os.unlink(dst)
        except OSError: pass
        os.symlink(os.path.split(ver)[1], dst)

    def copy_scons(self, src, dst, *args):
        try: os.unlink(dst)
        except OSError: pass
        self.copy_file(src, dst)
        self.outfiles.append(dst)

    def run(self):
        # --- distutils copy/paste ---
        if not self.skip_build:
            self.run_command('build_scripts')
        # --- /distutils copy/paste ---

        # Custom SCons installation stuff.
        if Options.hardlink_scons:
            create_basename_script = self.hardlink_scons
        elif Options.symlink_scons:
            create_basename_script = self.symlink_scons
        elif Options.install_scons_script:
            create_basename_script = self.copy_scons
        else:
            create_basename_script = self.do_nothing

        if Options.install_version_script:
            create_version_script = self.copy_scons
        else:
            create_version_script = self.do_nothing

        inputs = self.get_inputs()
        bat_scripts = [x for x in inputs if x[-4:] == '.bat']
        non_bat_scripts = [x for x in inputs if x[-4:] != '.bat']

        self.outfiles = []
        self.mkpath(self.install_dir)

        for src in non_bat_scripts:
            base = os.path.basename(src)
            scons = os.path.join(self.install_dir, base)
            scons_ver = scons + '-' + Version
            if is_win32:
                scons += '.py'
                scons_ver += '.py'
            create_version_script(src, scons_ver)
            create_basename_script(src, scons, scons_ver)

        if Options.install_bat:
            if is_win32:
                bat_install_dir = get_scons_prefix(self.install_dir, is_win32)
            else:
                bat_install_dir = self.install_dir
            for src in bat_scripts:
                scons_bat = os.path.join(bat_install_dir, 'scons.bat')
                scons_version_bat = os.path.join(bat_install_dir,
                                                 'scons-' + Version + '.bat')
                self.copy_scons(src, scons_bat)
                self.copy_scons(src, scons_version_bat)

        # --- distutils copy/paste ---
        if os.name == 'posix':
            # Set the executable bits (owner, group, and world) on
            # all the scripts we just installed.
            for file in self.get_outputs():
                if self.dry_run:
                    # log.info("changing mode of %s", file)
                    pass
                else:
                    # Use symbolic versions of permissions so this script doesn't fail to parse under python3.x
                    exec_and_read_permission = stat.S_IXOTH | stat.S_IXUSR | stat.S_IXGRP | stat.S_IROTH | stat.S_IRUSR | stat.S_IRGRP
                    mode_mask = 4095 # Octal 07777 used because python3 has different octal syntax than python 2
                    mode = ((os.stat(file)[stat.ST_MODE]) | exec_and_read_permission) & mode_mask
                    # log.info("changing mode of %s to %o", file, mode)
                    os.chmod(file, mode)
        # --- /distutils copy/paste ---

class build_scripts(_build_scripts):
    def finalize_options(self):
        _build_scripts.finalize_options(self)
        self.build_dir = os.path.join('build', 'scripts')

class install_data(_install_data):
    def initialize_options(self):
        _install_data.initialize_options(self)
    def finalize_options(self):
        _install_data.finalize_options(self)
        if force_to_usr_local(self):
            self.install_dir = '/usr/local'
        if Options.install_man:
            if is_win32:
                dir = 'Doc'
            else:
                dir = os.path.join('man', 'man1')
            self.data_files = [(dir, man_pages)]
            man_dir = os.path.join(self.install_dir, dir)
            msg = "Installed SCons man pages into %s" % man_dir
            Installed.append(msg)
        else:
            self.data_files = []

description = "Open Source next-generation build tool."

long_description = """Open Source next-generation build tool.
Improved, cross-platform substitute for the classic Make
utility.  In short, SCons is an easier, more reliable
and faster way to build software."""

scripts = [
    'script/scons',
    'script/sconsign',
    'script/scons-time',

    # We include scons.bat in the list of scripts, even on UNIX systems,
    # because we provide an option to allow it be installed explicitly,
    # for example if you're installing from UNIX on a share that's
    # accessible to Windows and you want the scons.bat.
    'script/scons.bat',
]

arguments = {
    'name'             : "scons",
    'version'          : Version,
    'description'      : description,
    'long_description' : long_description,
    'author'           : 'Steven Knight',
    'author_email'     : 'knight@baldmt.com',
    'url'              : "http://www.scons.org/",
    'packages'         : ["SCons",
                          "SCons.compat",
                          "SCons.Node",
                          "SCons.Options",
                          "SCons.Platform",
                          "SCons.Scanner",
                          "SCons.Script",
                          "SCons.Tool",
                          "SCons.Tool.docbook",
                          "SCons.Tool.MSCommon",
                          "SCons.Tool.packaging",
                          "SCons.Variables",
                         ],
    'package_dir'      : {'' : 'engine',
                          'SCons.Tool.docbook' : 'engine/SCons/Tool/docbook'},
    'package_data'     : {'SCons.Tool.docbook' : ['docbook-xsl-1.76.1/*',
                                                  'docbook-xsl-1.76.1/common/*',
                                                  'docbook-xsl-1.76.1/docsrc/*',
                                                  'docbook-xsl-1.76.1/eclipse/*',
                                                  'docbook-xsl-1.76.1/epub/*',
                                                  'docbook-xsl-1.76.1/epub/bin/*',
                                                  'docbook-xsl-1.76.1/epub/bin/lib/*',
                                                  'docbook-xsl-1.76.1/epub/bin/xslt/*',
                                                  'docbook-xsl-1.76.1/extensions/*',
                                                  'docbook-xsl-1.76.1/fo/*',
                                                  'docbook-xsl-1.76.1/highlighting/*',
                                                  'docbook-xsl-1.76.1/html/*',
                                                  'docbook-xsl-1.76.1/htmlhelp/*',
                                                  'docbook-xsl-1.76.1/images/*',
                                                  'docbook-xsl-1.76.1/images/callouts/*',
                                                  'docbook-xsl-1.76.1/images/colorsvg/*',
                                                  'docbook-xsl-1.76.1/javahelp/*',
                                                  'docbook-xsl-1.76.1/lib/*',
                                                  'docbook-xsl-1.76.1/manpages/*',
                                                  'docbook-xsl-1.76.1/params/*',
                                                  'docbook-xsl-1.76.1/profiling/*',
                                                  'docbook-xsl-1.76.1/roundtrip/*',
                                                  'docbook-xsl-1.76.1/slides/browser/*',
                                                  'docbook-xsl-1.76.1/slides/fo/*',
                                                  'docbook-xsl-1.76.1/slides/graphics/*',
                                                  'docbook-xsl-1.76.1/slides/graphics/active/*',
                                                  'docbook-xsl-1.76.1/slides/graphics/inactive/*',
                                                  'docbook-xsl-1.76.1/slides/graphics/toc/*',
                                                  'docbook-xsl-1.76.1/slides/html/*',
                                                  'docbook-xsl-1.76.1/slides/htmlhelp/*',
                                                  'docbook-xsl-1.76.1/slides/keynote/*',
                                                  'docbook-xsl-1.76.1/slides/keynote/xsltsl/*',
                                                  'docbook-xsl-1.76.1/slides/svg/*',
                                                  'docbook-xsl-1.76.1/slides/xhtml/*',
                                                  'docbook-xsl-1.76.1/template/*',
                                                  'docbook-xsl-1.76.1/tests/*',
                                                  'docbook-xsl-1.76.1/tools/bin/*',
                                                  'docbook-xsl-1.76.1/tools/make/*',
                                                  'docbook-xsl-1.76.1/webhelp/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/css/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/jquery/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/jquery/theme-redmond/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/jquery/theme-redmond/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/jquery/treeview/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/common/jquery/treeview/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/content/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/content/search/*',
                                                  'docbook-xsl-1.76.1/webhelp/docs/content/search/stemmers/*',
                                                  'docbook-xsl-1.76.1/webhelp/docsrc/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/css/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/jquery/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/jquery/theme-redmond/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/jquery/theme-redmond/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/jquery/treeview/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/common/jquery/treeview/images/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/content/search/*',
                                                  'docbook-xsl-1.76.1/webhelp/template/content/search/stemmers/*',
                                                  'docbook-xsl-1.76.1/webhelp/xsl/*',
                                                  'docbook-xsl-1.76.1/website/*',
                                                  'docbook-xsl-1.76.1/xhtml/*',
                                                  'docbook-xsl-1.76.1/xhtml-1_1/*',
                                                  'utils/*']},
    'data_files'       : [('man/man1', man_pages)],
    'scripts'          : scripts,
    'cmdclass'         : {'install'         : install,
                          'install_lib'     : install_lib,
                          'install_data'    : install_data,
                          'install_scripts' : install_scripts,
                          'build_scripts'   : build_scripts}
}

distutils.core.setup(**arguments)

if Installed:
    for i in Installed:
        print(i)

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



"""SCons.Script

This file implements the main() function used by the scons script.

Architecturally, this *is* the scons script, and will likely only be
called from the external "scons" wrapper.  Consequently, anything here
should not be, or be considered, part of the build engine.  If it's
something that we expect other software to want to use, it should go in
some other module.  If it's specific to the "scons" script invocation,
it goes here.

"""

#
# Copyright (c) 2001 - 2014 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "src/engine/SCons/Script/__init__.py  2014/09/27 12:51:43 garyo"

import time
start_time = time.time()

import collections
import os
import sys

# Special chicken-and-egg handling of the "--debug=memoizer" flag:
#
# SCons.Memoize contains a metaclass implementation that affects how
# the other classes are instantiated.  The Memoizer may add shim methods
# to classes that have methods that cache computed values in order to
# count and report the hits and misses.
#
# If we wait to enable the Memoization until after we've parsed the
# command line options normally, it will be too late, because the Memoizer
# will have already analyzed the classes that it's Memoizing and decided
# to not add the shims.  So we use a special-case, up-front check for
# the "--debug=memoizer" flag and enable Memoizer before we import any
# of the other modules that use it.

_args = sys.argv + os.environ.get('SCONSFLAGS', '').split()
if "--debug=memoizer" in _args:
    import SCons.Memoize
    import SCons.Warnings
    try:
        SCons.Memoize.EnableMemoization()
    except SCons.Warnings.Warning:
        # Some warning was thrown.  Arrange for it to be displayed
        # or not after warnings are configured.
        import Main
        exc_type, exc_value, tb = sys.exc_info()
        Main.delayed_warnings.append((exc_type, exc_value))
del _args

import SCons.Action
import SCons.Builder
import SCons.Environment
import SCons.Node.FS
import SCons.Options
import SCons.Platform
import SCons.Scanner
import SCons.SConf
import SCons.Subst
import SCons.Tool
import SCons.Util
import SCons.Variables
import SCons.Defaults

import Main

main                    = Main.main

# The following are global class definitions and variables that used to
# live directly in this module back before 0.96.90, when it contained
# a lot of code.  Some SConscript files in widely-distributed packages
# (Blender is the specific example) actually reached into SCons.Script
# directly to use some of these.  Rather than break those SConscript
# files, we're going to propagate these names into the SCons.Script
# namespace here.
#
# Some of these are commented out because it's *really* unlikely anyone
# used them, but we're going to leave the comment here to try to make
# it obvious what to do if the situation arises.
BuildTask               = Main.BuildTask
CleanTask               = Main.CleanTask
QuestionTask            = Main.QuestionTask
#PrintHelp               = Main.PrintHelp
#SConscriptSettableOptions = Main.SConscriptSettableOptions

AddOption               = Main.AddOption
GetOption               = Main.GetOption
SetOption               = Main.SetOption
Progress                = Main.Progress
GetBuildFailures        = Main.GetBuildFailures

#keep_going_on_error     = Main.keep_going_on_error
#print_dtree             = Main.print_dtree
#print_explanations      = Main.print_explanations
#print_includes          = Main.print_includes
#print_objects           = Main.print_objects
#print_time              = Main.print_time
#print_tree              = Main.print_tree
#memory_stats            = Main.memory_stats
#ignore_errors           = Main.ignore_errors
#sconscript_time         = Main.sconscript_time
#command_time            = Main.command_time
#exit_status             = Main.exit_status
#profiling               = Main.profiling
#repositories            = Main.repositories

#
import SConscript
_SConscript = SConscript

call_stack              = _SConscript.call_stack

#
Action                  = SCons.Action.Action
AddMethod               = SCons.Util.AddMethod
AllowSubstExceptions    = SCons.Subst.SetAllowableExceptions
Builder                 = SCons.Builder.Builder
Configure               = _SConscript.Configure
Environment             = SCons.Environment.Environment
#OptParser               = SCons.SConsOptions.OptParser
FindPathDirs            = SCons.Scanner.FindPathDirs
Platform                = SCons.Platform.Platform
Return                  = _SConscript.Return
Scanner                 = SCons.Scanner.Base
Tool                    = SCons.Tool.Tool
WhereIs                 = SCons.Util.WhereIs

#
BoolVariable            = SCons.Variables.BoolVariable
EnumVariable            = SCons.Variables.EnumVariable
ListVariable            = SCons.Variables.ListVariable
PackageVariable         = SCons.Variables.PackageVariable
PathVariable            = SCons.Variables.PathVariable

# Deprecated names that will go away some day.
BoolOption              = SCons.Options.BoolOption
EnumOption              = SCons.Options.EnumOption
ListOption              = SCons.Options.ListOption
PackageOption           = SCons.Options.PackageOption
PathOption              = SCons.Options.PathOption

# Action factories.
Chmod                   = SCons.Defaults.Chmod
Copy                    = SCons.Defaults.Copy
Delete                  = SCons.Defaults.Delete
Mkdir                   = SCons.Defaults.Mkdir
Move                    = SCons.Defaults.Move
Touch                   = SCons.Defaults.Touch

# Pre-made, public scanners.
CScanner                = SCons.Tool.CScanner
DScanner                = SCons.Tool.DScanner
DirScanner              = SCons.Defaults.DirScanner
ProgramScanner          = SCons.Tool.ProgramScanner
SourceFileScanner       = SCons.Tool.SourceFileScanner

# Functions we might still convert to Environment methods.
CScan                   = SCons.Defaults.CScan
DefaultEnvironment      = SCons.Defaults.DefaultEnvironment

# Other variables we provide.
class TargetList(collections.UserList):
    def _do_nothing(self, *args, **kw):
        pass
    def _add_Default(self, list):
        self.extend(list)
    def _clear(self):
        del self[:]

ARGUMENTS               = {}
ARGLIST                 = []
BUILD_TARGETS           = TargetList()
COMMAND_LINE_TARGETS    = []
DEFAULT_TARGETS         = []

# BUILD_TARGETS can be modified in the SConscript files.  If so, we
# want to treat the modified BUILD_TARGETS list as if they specified
# targets on the command line.  To do that, though, we need to know if
# BUILD_TARGETS was modified through "official" APIs or by hand.  We do
# this by updating two lists in parallel, the documented BUILD_TARGETS
# list, above, and this internal _build_plus_default targets list which
# should only have "official" API changes.  Then Script/Main.py can
# compare these two afterwards to figure out if the user added their
# own targets to BUILD_TARGETS.
_build_plus_default = TargetList()

def _Add_Arguments(alist):
    for arg in alist:
        a, b = arg.split('=', 1)
        ARGUMENTS[a] = b
        ARGLIST.append((a, b))

def _Add_Targets(tlist):
    if tlist:
        COMMAND_LINE_TARGETS.extend(tlist)
        BUILD_TARGETS.extend(tlist)
        BUILD_TARGETS._add_Default = BUILD_TARGETS._do_nothing
        BUILD_TARGETS._clear = BUILD_TARGETS._do_nothing
        _build_plus_default.extend(tlist)
        _build_plus_default._add_Default = _build_plus_default._do_nothing
        _build_plus_default._clear = _build_plus_default._do_nothing

def _Set_Default_Targets_Has_Been_Called(d, fs):
    return DEFAULT_TARGETS

def _Set_Default_Targets_Has_Not_Been_Called(d, fs):
    if d is None:
        d = [fs.Dir('.')]
    return d

_Get_Default_Targets = _Set_Default_Targets_Has_Not_Been_Called

def _Set_Default_Targets(env, tlist):
    global DEFAULT_TARGETS
    global _Get_Default_Targets
    _Get_Default_Targets = _Set_Default_Targets_Has_Been_Called
    for t in tlist:
        if t is None:
            # Delete the elements from the list in-place, don't
            # reassign an empty list to DEFAULT_TARGETS, so that the
            # variables will still point to the same object we point to.
            del DEFAULT_TARGETS[:]
            BUILD_TARGETS._clear()
            _build_plus_default._clear()
        elif isinstance(t, SCons.Node.Node):
            DEFAULT_TARGETS.append(t)
            BUILD_TARGETS._add_Default([t])
            _build_plus_default._add_Default([t])
        else:
            nodes = env.arg2nodes(t, env.fs.Entry)
            DEFAULT_TARGETS.extend(nodes)
            BUILD_TARGETS._add_Default(nodes)
            _build_plus_default._add_Default(nodes)

#
help_text = None

def HelpFunction(text):
    global help_text
    if SCons.Script.help_text is None:
        SCons.Script.help_text = text
    else:
        help_text = help_text + text

#
# Will be non-zero if we are reading an SConscript file.
sconscript_reading = 0

#
def Variables(files=[], args=ARGUMENTS):
    return SCons.Variables.Variables(files, args)

def Options(files=[], args=ARGUMENTS):
    return SCons.Options.Options(files, args)

# The list of global functions to add to the SConscript name space
# that end up calling corresponding methods or Builders in the
# DefaultEnvironment().
GlobalDefaultEnvironmentFunctions = [
    # Methods from the SConsEnvironment class, above.
    'Default',
    'EnsurePythonVersion',
    'EnsureSConsVersion',
    'Exit',
    'Export',
    'GetLaunchDir',
    'Help',
    'Import',
    #'SConscript', is handled separately, below.
    'SConscriptChdir',

    # Methods from the Environment.Base class.
    'AddPostAction',
    'AddPreAction',
    'Alias',
    'AlwaysBuild',
    'BuildDir',
    'CacheDir',
    'Clean',
    #The Command() method is handled separately, below.
    'Decider',
    'Depends',
    'Dir',
    'NoClean',
    'NoCache',
    'Entry',
    'Execute',
    'File',
    'FindFile',
    'FindInstalledFiles',
    'FindSourceFiles',
    'Flatten',
    'GetBuildPath',
    'Glob',
    'Ignore',
    'Install',
    'InstallAs',
    'Literal',
    'Local',
    'ParseDepends',
    'Precious',
    'Repository',
    'Requires',
    'SConsignFile',
    'SideEffect',
    'SourceCode',
    'SourceSignatures',
    'Split',
    'Tag',
    'TargetSignatures',
    'Value',
    'VariantDir',
]

GlobalDefaultBuilders = [
    # Supported builders.
    'CFile',
    'CXXFile',
    'DVI',
    'Jar',
    'Java',
    'JavaH',
    'Library',
    'M4',
    'MSVSProject',
    'Object',
    'PCH',
    'PDF',
    'PostScript',
    'Program',
    'RES',
    'RMIC',
    'SharedLibrary',
    'SharedObject',
    'StaticLibrary',
    'StaticObject',
    'Tar',
    'TypeLibrary',
    'Zip',
    'Package',
]

for name in GlobalDefaultEnvironmentFunctions + GlobalDefaultBuilders:
    exec "%s = _SConscript.DefaultEnvironmentCall(%s)" % (name, repr(name))
del name

# There are a handful of variables that used to live in the
# Script/SConscript.py module that some SConscript files out there were
# accessing directly as SCons.Script.SConscript.*.  The problem is that
# "SConscript" in this namespace is no longer a module, it's a global
# function call--or more precisely, an object that implements a global
# function call through the default Environment.  Nevertheless, we can
# maintain backwards compatibility for SConscripts that were reaching in
# this way by hanging some attributes off the "SConscript" object here.
SConscript = _SConscript.DefaultEnvironmentCall('SConscript')

# Make SConscript look enough like the module it used to be so
# that pychecker doesn't barf.
SConscript.__name__ = 'SConscript'

SConscript.Arguments = ARGUMENTS
SConscript.ArgList = ARGLIST
SConscript.BuildTargets = BUILD_TARGETS
SConscript.CommandLineTargets = COMMAND_LINE_TARGETS
SConscript.DefaultTargets = DEFAULT_TARGETS

# The global Command() function must be handled differently than the
# global functions for other construction environment methods because
# we want people to be able to use Actions that must expand $TARGET
# and $SOURCE later, when (and if) the Action is invoked to build
# the target(s).  We do this with the subst=1 argument, which creates
# a DefaultEnvironmentCall instance that wraps up a normal default
# construction environment that performs variable substitution, not a
# proxy that doesn't.
#
# There's a flaw here, though, because any other $-variables on a command
# line will *also* be expanded, each to a null string, but that should
# only be a problem in the unusual case where someone was passing a '$'
# on a command line and *expected* the $ to get through to the shell
# because they were calling Command() and not env.Command()...  This is
# unlikely enough that we're going to leave this as is and cross that
# bridge if someone actually comes to it.
Command = _SConscript.DefaultEnvironmentCall('Command', subst=1)

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:
